package com.smcic.api.conference.service.impl;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.ImmutableMap;
import com.smcic.api.conference.dto.WxPhoneDTO;
import com.smcic.api.conference.entity.ConferenceWxAccount;
import com.smcic.api.conference.entity.UserThird;
import com.smcic.api.conference.mapper.UserThirdMapper;
import com.smcic.api.conference.service.IUserThirdService;
import com.smcic.api.operate.entity.KeysConst;
import com.smcic.core.advice.APIException;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.util.TextUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author syj
 * @since 2024-03-27
 */
@Service
@Slf4j
public class UserThirdServiceImpl extends ServiceImpl<UserThirdMapper, UserThird> implements IUserThirdService {

    private Map<Long, String> access_token = new ConcurrentHashMap<>();

    @Resource
    private ConferenceWxAccountServiceImpl conferenceWxAccountService;

    @Resource
    private RedisTemplate<String, String> redisTemplate;


    public String  getAccessToken(String appid, String secret) {
        // 1.拼接请求地址
        String url = "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=" + appid + "&secret=" + secret;
        // 2.发送请求,获取响应数据
        String response = HttpUtil.get(url);
        // 3.提取access_token
        Map<String, String> map = JSON.parseObject(response, new TypeReference<Map<String, String>>(){});
        return map.get("access_token");
    }

    public String getPhone(Long cid, String code){
        String phone = redisTemplate.opsForValue().get("WX_CODE_" + code);
        if(!TextUtils.isEmpty(phone)){
            return phone; // 从缓存中获取
        }
        ConferenceWxAccount wxAccount = conferenceWxAccountService.getById(cid);
        if (wxAccount == null) {
            throw new APIException("请先配置会议微信账号");
        }
        String token;
        if (!access_token.containsKey(cid)){
            token = getAccessToken(wxAccount.getWxAppid(), wxAccount.getWxSecret());
            access_token.put(cid, token);
        }else{
            token =  access_token.get(cid);
        }
        String url = "https://api.weixin.qq.com/wxa/business/getuserphonenumber?access_token=" + token;
        String response = HttpUtil.post(url, JSON.toJSONString(ImmutableMap.of("code", code)));
        WxPhoneDTO wxPhoneDTO = JSON.parseObject(response, WxPhoneDTO.class);
        log.info("获取wx手机号{}", response);
        if (wxPhoneDTO.getErrcode() != 0){
            throw new APIException("获取用户手机号失败");
        }
        redisTemplate.opsForValue().set("WX_CODE_" + code, wxPhoneDTO.getPhoneInfo().getPhoneNumber(), 60 * 60 * 24 * 30);  // 30天过期
        return wxPhoneDTO.getPhoneInfo().getPhoneNumber();
    }

    public void allRefresh(){
        for (ConferenceWxAccount wxAccount  : conferenceWxAccountService.list()){
            String token = getAccessToken(wxAccount.getWxAppid(), wxAccount.getWxSecret());
            access_token.put(wxAccount.getConferenceId(), token);
        }
    }

}
