package com.smcic.api.operate.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.lang.generator.SnowflakeGenerator;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.smcic.api.operate.entity.*;
import com.smcic.api.operate.mapper.VoteInfoMapper;
import com.smcic.api.operate.service.IVoteInfoService;
import com.smcic.core.advice.APIException;
import com.smcic.core.auth.HttpContextUtil;
import org.apache.http.util.TextUtils;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Map;
import java.util.Optional;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author syj
 * @since 2022-12-16
 */
@Service
public class VoteInfoServiceImpl extends ServiceImpl<VoteInfoMapper, VoteInfo> implements IVoteInfoService {

    @Resource
    private EnrollInfoServiceImpl enrollInfoService;

    @Resource
    private OperateServiceImpl operateService;

    @Resource
    private RedisTemplate<String, String> redisTemplate;

    private final SnowflakeGenerator snowflakeGenerator = new SnowflakeGenerator(1, 1);

    @Resource
    private KafkaTemplate kafkaTemplate;

    @Resource
    private ObjectMapper objectMapper;

    private final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyyMMdd");

    public void vote(String target, String client, Integer operateId){
        Optional<UserAuth> user = HttpContextUtil.getUser();
        String source = user.get().getUid();

        String dt = formatter.format(LocalDate.now());
        operateService.verify(operateId);
        long max = 2L;
        //client.equals("闪视频")
        if(UserPlatform.SSP.equals(user.get().getPlatform())){
            max = 10L;
        }

        String one = redisTemplate.opsForValue().get(KeysConst.TARGET + operateId + "_" + target);

        if (null == one){
            throw new APIException(400, "不存在的目标");
        }

        String vds = redisTemplate.opsForValue().get(KeysConst.SOURCE + operateId + "_"+ dt + "_" + source);
        long vd;
        if(TextUtils.isEmpty(vds)){
            vd = 0L;
        }else{
            vd = Long.parseLong(vds);
        }

        if (vd >= max){
            if(UserPlatform.SSP.equals(user.get().getPlatform())){
                throw new APIException(400, "今日投票次数已用完，请明日再来!");
            }else{
                throw new APIException(400, "今日投票次数已用完，点击下载闪视频获取更多投票机会!");
            }
        }

        long id = snowflakeGenerator.next();

        String lockKey = KeysConst.VOTE_LOCK + operateId + "_" + source;
        while (true){
            Boolean success = redisTemplate.opsForValue().setIfAbsent(lockKey, String.valueOf(id), Duration.ofSeconds(3));

            if (Boolean.TRUE.equals(success)){
                vds = redisTemplate.opsForValue().get(KeysConst.SOURCE + operateId + "_"+ dt + "_" + source);
                if(TextUtils.isEmpty(vds)){
                    vd = 0L;
                }else{
                    vd = Long.parseLong(vds);
                }
                if (vd >= max){
                    redisTemplate.delete(lockKey);
                    if(UserPlatform.SSP.equals(user.get().getPlatform())){
                        throw new APIException(400, "今日投票次数已用完，请明日再来!");
                    }else{
                        throw new APIException(400, "今日投票次数已用完，点击下载闪视频获取更多投票机会!");
                    }
                }

                try {
                    kafkaTemplate.send(KeysConst.KAFKA_QUEUE,
                            objectMapper.writeValueAsString(
                                    new VoteInfo(target, source, client, LocalDateTime.now(), LocalDate.now())
                    ));
                } catch (JsonProcessingException e) {
                    throw new RuntimeException(e);
                }
                redisTemplate.opsForValue().increment(KeysConst.SOURCE + operateId + "_"+ dt + "_" + source);
                redisTemplate.expire(KeysConst.SOURCE + operateId + "_"+ dt + "_" + source, Duration.ofDays(1));
                redisTemplate.delete(lockKey);
                break;
            }
        }

    }

    public void lsVerify(Integer operateId, String phone){
        if(!TextUtils.isEmpty(phone) && votedTimes(operateId, phone) < 5){
            throw new APIException("您暂无查看权限");
        }
    }

    public Long votedTimes(Integer operateId, String phone){
        String source = redisTemplate.opsForValue().get(KeysConst.SOURCE + operateId + "_" + formatter.format(LocalDate.now()) + "_" + phone);
        if(TextUtils.isEmpty(source))
            return 0L;
        return Long.valueOf(source);
    }

}
